@@ -39,7 +39,7 @@ spectralG.cpp <- function(ZETA, ZWs = NULL, X = NULL, weights = 1, return.G = TR
   if((!is.null(tol)) & (length(tol) == 1)){
     tol <- rep(tol, 2)
   }
-
+  
   lines.name.pheno <- rownames(ZETA[[1]]$Z)
   n <- nrow(ZETA[[1]]$Z)
   ms.ZETA <- unlist(lapply(ZETA, function(x) ncol(x$Z)))
@@ -54,21 +54,21 @@ spectralG.cpp <- function(ZETA, ZWs = NULL, X = NULL, weights = 1, return.G = TR
     ms.ZWs <- NULL
     m.ZWs <- 0
   }
-
+  
   m <- m.ZETA + m.ZWs
-
+  
   if((lz + lw) != length(weights)){
     stop("Weights should have the same length as ZETA!")
   }
   stopifnot(all(weights >= 0))
-
+  
   if(is.null(X)){
     X <- as.matrix(rep(1, n))
     rownames(X) <- lines.name.pheno
     colnames(X) <- "Intercept"
   }
   p <- ncol(X)
-
+  
   if(is.null(spectral.method)){
     if(n <= m + p){
       spectral.method <- "eigen"
@@ -76,32 +76,32 @@ spectralG.cpp <- function(ZETA, ZWs = NULL, X = NULL, weights = 1, return.G = TR
       spectral.method <- "cholesky"
     }
   }
-
+  
   if(is.null(df.H)){
     df.H <- n
   }
-
+  
   if(spectral.method == "cholesky"){
     ZBt <- NULL
     for(i in 1:lz){
       Z.now <- ZETA[[i]]$Z
       K.now <- ZETA[[i]]$K
-
+      
       diag(K.now) <- diag(K.now) + 1e-06
       B.now <- try(chol(K.now), silent = TRUE)
       if ("try-error" %in% class(B.now)) {
         stop("K not positive semi-definite.")
       }
-
+      
       ZBt.now <- tcrossprod(Z.now, B.now)
       ZBt <- cbind(ZBt, sqrt(weights[i]) * ZBt.now)
     }
-
+    
     if(!is.null(ZWs)){
       for(j in 1:lw){
         Z.now <- ZWs[[j]]$Z
         Bt.now <- ZWs[[j]]$W %*% ZWs[[j]]$Gamma
-
+        
         ZBt.now <- Z.now %*% Bt.now
         ZBt <- cbind(ZBt, sqrt(weights[lz + j]) * ZBt.now)
       }
@@ -109,13 +109,13 @@ spectralG.cpp <- function(ZETA, ZWs = NULL, X = NULL, weights = 1, return.G = TR
     spectral_G.res <- try(spectralG_cholesky(zbt = ZBt, x = X, return_G = return.G,
                                              return_SGS = return.SGS),
                           silent = TRUE)
-
+    
     if(!("try-error" %in% class(spectral_G.res))){
-
+      
       if(return.G){
         U0 <- spectral_G.res$U
         delta0 <- c(spectral_G.res$D)
-
+        
         if(!is.null(tol)){
           G.tol.over <- which(delta0 > tol[2])
         }else{
@@ -124,11 +124,11 @@ spectralG.cpp <- function(ZETA, ZWs = NULL, X = NULL, weights = 1, return.G = TR
         delta <- delta0[G.tol.over]
         U <- U0[, G.tol.over]
       }
-
+      
       if(return.SGS){
         Q0 <- spectral_G.res$Q
         theta0 <- c(spectral_G.res$theta)
-
+        
         if(!is.null(tol)){
           SGS.tol.over <- which(theta0 > tol[2])
         }else{
@@ -141,34 +141,34 @@ spectralG.cpp <- function(ZETA, ZWs = NULL, X = NULL, weights = 1, return.G = TR
       spectral.method <- "eigen"
     }
   }
-
-
+  
+  
   if(spectral.method == "eigen"){
     ZKZt <- matrix(0, nrow = n, ncol = n)
     for(i in 1:lz){
       Z.now <- ZETA[[i]]$Z
       K.now <- ZETA[[i]]$K
-
+      
       ZKZt.now <- tcrossprod(Z.now %*% K.now, Z.now)
       ZKZt <- ZKZt + weights[i] * ZKZt.now
     }
-
+    
     if(!is.null(ZWs)){
       for(j in 1:lw){
         Z.now <- ZWs[[j]]$Z
         K.now <- tcrossprod(ZWs[[j]]$W %*% ZWs[[j]]$Gamma, ZWs[[j]]$W)
-
+        
         ZKZt.now <- tcrossprod(Z.now %*% K.now, Z.now)
         ZKZt <- ZKZt + weights[lz + j] * ZKZt.now
       }
     }
     spectral_G.res <- spectralG_eigen(zkzt = ZKZt, x = X, return_G = return.G,
                                       return_SGS = return.SGS)
-
+    
     if(return.G){
       U0 <- spectral_G.res$U
       delta0 <- c(spectral_G.res$D)
-
+      
       if(!is.null(tol)){
         G.tol.over <- which(delta0 > tol[2])
       }else{
@@ -177,11 +177,11 @@ spectralG.cpp <- function(ZETA, ZWs = NULL, X = NULL, weights = 1, return.G = TR
       delta <- delta0[G.tol.over]
       U <- U0[, G.tol.over]
     }
-
+    
     if(return.SGS){
       Q0 <- spectral_G.res$Q
       theta0 <- c(spectral_G.res$theta)
-
+      
       if(!is.null(tol)){
         SGS.tol.over <- which(theta0 > tol[2])
       }else{
@@ -191,21 +191,21 @@ spectralG.cpp <- function(ZETA, ZWs = NULL, X = NULL, weights = 1, return.G = TR
       Q <- Q0[, SGS.tol.over]
     }
   }
-
-
+  
+  
   if(return.G){
     spectral.G <- list(U = U, delta = delta)
   }else{
     spectral.G <- NULL
   }
-
+  
   if(return.SGS){
     spectral.SGS <- list(Q = Q, theta = theta)
   }else{
     spectral.SGS <- NULL
   }
-
-
+  
+  
   return(list(spectral.G = spectral.G,
               spectral.SGS = spectral.SGS))
 }
@@ -238,6 +238,7 @@ spectralG.cpp <- function(ZETA, ZWs = NULL, X = NULL, weights = 1, return.G = TR
 #' The result of the eigen decompsition of \eqn{G = ZKZ'}. You can use "spectralG.cpp" function in RAINBOWR.
 #' If this argument is NULL, the eigen decomposition will be performed in this function.
 #' We recommend you assign the result of the eigen decomposition beforehand for time saving.
+#' @param n.core Setting n.core > 1 will enable parallel execution on a machine with multiple cores.
 #' @param lam.len The number of initial values you set. If this number is large, the estimation will be more accurate,
 #' but computational cost will be large. We recommend setting this value 3 <= lam.len <= 6.
 #' @param init.range The range of the initial parameters. For example, if lam.len = 5 and init.range = c(1e-06, 1e02),
@@ -287,26 +288,26 @@ spectralG.cpp <- function(ZETA, ZWs = NULL, X = NULL, weights = 1, return.G = TR
 #'
 #'
 #'
-EMM1.cpp <- function(y, X = NULL, ZETA, eigen.G = NULL, lam.len = 4, init.range = c(1e-04, 1e02),
-                     init.one = 0.5, conv.param = 1e-06, count.max = 15, bounds = c(1e-06, 1e06),
-                     tol = NULL, REML = TRUE, silent = TRUE, plot.l = FALSE,
-                     SE = FALSE, return.Hinv = TRUE){
-
+EMM1.cpp <- function(y, X = NULL, ZETA, eigen.G = NULL, n.core = NA, lam.len = 4, 
+                     init.range = c(1e-04, 1e02), init.one = 0.5, conv.param = 1e-06,
+                     count.max = 15, bounds = c(1e-06, 1e06), tol = NULL, REML = TRUE,
+                     silent = TRUE, plot.l = FALSE, SE = FALSE, return.Hinv = TRUE){
+  
   #### The start of EMM1 (GEMMA-based) ####
   if(length(ZETA) != 1){
     stop("If you want to solve multikernel mixed-model equation, you should use EM3.cpp function.")
   }
-
+  
   ### Some definitions ###
   y0 <- as.matrix(y)
   n0 <- length(y0)
-
+  
   if (is.null(X)) {
     p <- 1
     X <- matrix(rep(1, n0), n0, 1)
   }
   p <- ncol(X)
-
+  
   Z <- ZETA[[1]]$Z
   if (is.null(Z)) {
     Z <- diag(n0)
@@ -316,11 +317,11 @@ EMM1.cpp <- function(y, X = NULL, ZETA, eigen.G = NULL, lam.len = 4, init.range
     m <- 1
     Z <- matrix(Z, length(Z), 1)
   }
-
-
+  
+  
   stopifnot(nrow(Z) == n0)
   stopifnot(nrow(X) == n0)
-
+  
   K <- ZETA[[1]]$K
   if (!is.null(K)) {
     stopifnot(nrow(K) == m)
@@ -331,20 +332,21 @@ EMM1.cpp <- function(y, X = NULL, ZETA, eigen.G = NULL, lam.len = 4, init.range
   X <- X[not.NA, , drop = FALSE]
   n <- length(not.NA)
   y <- matrix(y0[not.NA], n, 1)
-
+  
   spI <- diag(n)
   logXtX <- sum(log(eigen(crossprod(X), symmetric = TRUE)$values))
-
+  
   ### Decide initial parameter and calculate H ###
-  if(lam.len >= 2){
+  if (lam.len >= 2) {
     h2.0 <- seq(init.range[1], 1 - 1 / init.range[2], length = lam.len)
-    #lambda.0 <- h2.0 / (1 - h2.0)
-    lambda.0 <- 10 ^ seq(log10(init.range[1]), log10(init.range[2]), length = lam.len)
-  }else{
+    # lambda.0 <- h2.0 / (1 - h2.0)
+    lambda.0 <-
+      10 ^ seq(log10(init.range[1]), log10(init.range[2]), length = lam.len)
+  } else {
     lambda.0 <- init.one
   }
-
-
+  
+  
   ### Eigen decomposition of ZKZt ###
   if(is.null(eigen.G)){
     eigen.G <- spectralG.cpp(ZETA = lapply(ZETA, function(x) list(Z = x$Z[not.NA, , drop = FALSE], K = x$K)),
@@ -352,14 +354,23 @@ EMM1.cpp <- function(y, X = NULL, ZETA, eigen.G = NULL, lam.len = 4, init.range
   }
   U <- eigen.G$U
   delta <- eigen.G$delta
-
-
+  
+  
   if(lam.len >= 2){
     it <- split(lambda.0, factor(1:lam.len))
-    mclap.res <- unlist(parallel::mclapply(it, ml_est_out, y = as.matrix(y), x = as.matrix(X),
+    
+    if (is.na(n.core)) {
+      n.core <- lam.len
+    } else {
+      stopifnot(is.numeric(n.core))
+      stopifnot(n.core >= 1)
+      n.core <- floor(n.core)
+    }
+    
+    mclap.res <- unlist(parallel::mclapply(X = it, FUN = ml_est_out, y = as.matrix(y), x = as.matrix(X),
                                            u = U, delta = as.matrix(delta), z = Z, k = K,
                                            logXtX = logXtX, conv_param = conv.param, count_max = count.max,
-                                           bounds1 = bounds[1], bounds2 = bounds[2], REML = REML, mc.cores = lam.len))
+                                           bounds1 = bounds[1], bounds2 = bounds[2], REML = REML, mc.cores = n.core))
     counts <- mclap.res[seq(3, 3 * lam.len, by = 3)]
     l.maxs <- mclap.res[seq(2, 3 * lam.len - 1, by = 3)]
     lambda.opts <- mclap.res[seq(1, 3 * lam.len - 2, by = 3)]
@@ -375,22 +386,22 @@ EMM1.cpp <- function(y, X = NULL, ZETA, eigen.G = NULL, lam.len = 4, init.range
     counts <- ml.res$count
     names(lambda.opt) <- "lambda.opt"
   }
-
-
-  if(all(counts[-1] == count.max)){
-    if(!silent){
+  
+  
+  if (all(counts[-1] == count.max)) {
+    if (!silent) {
       warning("Parameter estimation may not be accurate. Please reestimate with EMM2.cpp function.")
     }
     reest <- 1
-  }else{
+  } else {
     reest <- 0
   }
-
+  
   res.list <- ml_one_step(lambda.opt, y = as.matrix(y), x = as.matrix(X),
                           u = U, delta = as.matrix(delta), z = Z, k = K,
                           logXtX = logXtX, conv_param = conv.param, count_max = count.max,
                           bounds1 = bounds[1], bounds2 = bounds[2], REML = REML, SE = SE, return_Hinv = return.Hinv)
-
+  
   return(c(res.list, list(lambda = 1 / lambda.opt), list(lambdas = 1 / lambda.opts),
            list(reest = reest), list(counts = counts)))
 }
@@ -401,6 +412,7 @@ EMM1.cpp <- function(y, X = NULL, ZETA, eigen.G = NULL, lam.len = 4, init.range
 
 
 
+
 #' Equation of mixed model for one kernel, EMMA-based method (inplemented by Rcpp)
 #'
 #' @description This function solves single-kernel linear mixed model by EMMA
@@ -459,22 +471,22 @@ EMM1.cpp <- function(y, X = NULL, ZETA, eigen.G = NULL, lam.len = 4, init.range
 #'
 EMM2.cpp <- function(y, X = NULL, ZETA, eigen.G = NULL, eigen.SGS = NULL, tol = NULL, optimizer = "nlminb",
                      traceInside = 0, REML = TRUE, bounds = c(1e-09, 1e+09), SE = FALSE, return.Hinv = FALSE){
-
+  
   #### The start of EMM2 (EMMA-based) ####
   if(length(ZETA) != 1){
     stop("If you want to solve multikernel mixed-model equation, you should use EM3.cpp function.")
   }
-
+  
   pi <- 3.14159
   y0 <- as.matrix(y)
   n0 <- length(y0)
-
+  
   if (is.null(X)) {
     p <- 1
     X <- matrix(rep(1, n0), n0, 1)
   }
   p <- ncol(X)
-
+  
   Z <- ZETA[[1]]$Z
   if (is.null(Z)) {
     Z <- diag(n0)
@@ -484,11 +496,11 @@ EMM2.cpp <- function(y, X = NULL, ZETA, eigen.G = NULL, eigen.SGS = NULL, tol =
     m <- 1
     Z <- matrix(Z, length(Z), 1)
   }
-
-
+  
+  
   stopifnot(nrow(Z) == n0)
   stopifnot(nrow(X) == n0)
-
+  
   K <- ZETA[[1]]$K
   if (!is.null(K)) {
     stopifnot(nrow(K) == m)
@@ -499,9 +511,9 @@ EMM2.cpp <- function(y, X = NULL, ZETA, eigen.G = NULL, eigen.SGS = NULL, tol =
   X <- X[not.NA, , drop = FALSE]
   n <- length(not.NA)
   y <- matrix(y0[not.NA], n, 1)
-
+  
   spI <- diag(n)
-
+  
   ### Eigen decomposition of ZKZt ###
   if(is.null(eigen.G)){
     eigen.G <- spectralG.cpp(ZETA = lapply(ZETA, function(x) list(Z = x$Z[not.NA, , drop = FALSE], K = x$K)),
@@ -509,24 +521,24 @@ EMM2.cpp <- function(y, X = NULL, ZETA, eigen.G = NULL, eigen.SGS = NULL, tol =
   }
   U <- eigen.G$U
   delta <- eigen.G$delta
-
+  
   if(is.null(eigen.SGS)){
     eigen.SGS <- spectralG.cpp(ZETA = lapply(ZETA, function(x) list(Z = x$Z[not.NA, , drop = FALSE], K = x$K)),
                                X = X, return.G = FALSE, return.SGS = TRUE, tol = tol, df.H = NULL)[[2]]
   }
   Q <- eigen.SGS$Q
   theta <- eigen.SGS$theta
-
+  
   omega <- crossprod(Q, y)
   omega.sq <- omega ^ 2
-
+  
   lambda.0 <- 1
   n <- nrow(X)
   p <- ncol(X)
   phi <- delta
   traceNo <- ifelse(traceInside > 0, 3, 0)
   traceREPORT <- ifelse(traceInside > 0, traceInside, 1)
-
+  
   if (!REML) {
     f.ML <- function(lambda, n, theta, omega.sq, phi) {
       n * log(sum(omega.sq / (theta + lambda))) + sum(log(phi +
@@ -536,7 +548,7 @@ EMM2.cpp <- function(y, X = NULL, ZETA, eigen.G = NULL, eigen.SGS = NULL, tol =
     #   n * sum(omega.sq / (theta + lambda) ^ 2) / sum(omega.sq / (theta + lambda)) -
     #     sum(1 / (phi + lambda))
     # }
-
+    
     if (optimizer == "optim") {
       soln <- optimize(f.ML, interval = bounds, n, theta,
                        omega.sq, phi)
@@ -564,7 +576,7 @@ EMM2.cpp <- function(y, X = NULL, ZETA, eigen.G = NULL, eigen.SGS = NULL, tol =
       lambda.opt <- soln$minimum
       maxval <- soln$objective
     }
-
+    
     df <- n
   } else {
     f.REML <- function(lambda, n.p, theta, omega.sq) {
@@ -574,7 +586,7 @@ EMM2.cpp <- function(y, X = NULL, ZETA, eigen.G = NULL, eigen.SGS = NULL, tol =
     #   n.p * sum(omega.sq / (theta + lambda) ^ 2) / sum(omega.sq / (theta + lambda)) -
     #     sum(1 / (theta + lambda))
     # }
-
+    
     if (optimizer == "optim") {
       soln <- optimize(f.REML, interval = bounds, n - p, theta,
                        omega.sq)
@@ -601,11 +613,11 @@ EMM2.cpp <- function(y, X = NULL, ZETA, eigen.G = NULL, eigen.SGS = NULL, tol =
       lambda.opt <- soln$minimum
       maxval <- soln$objective
     }
-
+    
     df <- n - p
   }
-
-
+  
+  
   EMM2.final.res <- EMM2_last_step(lambda.opt, as.matrix(y), X, U, as.matrix(delta),
                                    Z, K, as.matrix(theta), as.matrix(omega.sq),
                                    as.matrix(phi), maxval, n, p, df, SE, return.Hinv)
@@ -650,6 +662,7 @@ EMM2.cpp <- function(y, X = NULL, ZETA, eigen.G = NULL, eigen.SGS = NULL, tol =
 #' We recommend you assign the result of the eigen decomposition beforehand for time saving.
 #' @param n.thres If \eqn{n >= n.thres}, perform EMM1.cpp. Else perform EMM2.cpp.
 #' @param reestimation If TRUE, EMM2.cpp is performed when the estimation by EMM1.cpp may not be accurate.
+#' @param n.core Setting n.core > 1 will enable parallel execution on a machine with multiple cores.
 #' @param lam.len The number of initial values you set. If this number is large, the estimation will be more accurate,
 #' but computational cost will be large. We recommend setting this value 3 <= lam.len <= 6.
 #' @param init.range The range of the initial parameters. For example, if lam.len = 5 and init.range = c(1e-06, 1e02),
@@ -705,28 +718,29 @@ EMM2.cpp <- function(y, X = NULL, ZETA, eigen.G = NULL, eigen.SGS = NULL, tol =
 #'
 #'
 #'
-EMM.cpp <- function(y, X = NULL, ZETA, eigen.G = NULL, eigen.SGS = NULL, n.thres = 450, reestimation = FALSE,
-                    lam.len = 4, init.range = c(1e-06, 1e02), init.one = 0.5, conv.param = 1e-06,
-                    count.max = 20, bounds = c(1e-06, 1e06), tol = NULL, optimizer = "nlminb", traceInside = 0, REML = TRUE,
+EMM.cpp <- function(y, X = NULL, ZETA, eigen.G = NULL, eigen.SGS = NULL, n.thres = 450,
+                    reestimation = FALSE, n.core = NA, lam.len = 4, init.range = c(1e-06, 1e02),
+                    init.one = 0.5, conv.param = 1e-06, count.max = 20, bounds = c(1e-06, 1e06),
+                    tol = NULL, optimizer = "nlminb", traceInside = 0, REML = TRUE,
                     silent = TRUE, plot.l = FALSE, SE = FALSE, return.Hinv = TRUE){
   n <- length(y)
-
+  
   if(n >= n.thres){
-    res <- EMM1.cpp(y = y, X = X, ZETA = ZETA, eigen.G = eigen.G, lam.len = lam.len,
+    res <- EMM1.cpp(y = y, X = X, ZETA = ZETA, eigen.G = eigen.G, n.core = n.core, lam.len = lam.len,
                     init.range = init.range, init.one = init.one, conv.param = conv.param,
                     count.max = count.max, bounds = bounds, tol = tol, REML = REML,
                     silent = silent, plot.l = plot.l, SE = SE, return.Hinv = return.Hinv)
-
+    
     if((res$reest == 1) & reestimation){
       res <- EMM2.cpp(y = y, X = X, ZETA = ZETA, eigen.G = eigen.G, eigen.SGS = eigen.SGS, REML = REML,
                       optimizer = optimizer, traceInside = traceInside, tol = tol, bounds = bounds, SE = SE, return.Hinv = return.Hinv)
     }
-
+    
   }else{
     res <- EMM2.cpp(y = y, X = X, ZETA = ZETA, eigen.G = eigen.G, eigen.SGS = eigen.SGS, REML = REML,
                     optimizer = optimizer, traceInside = traceInside, bounds = bounds, SE = SE, return.Hinv = return.Hinv, tol = tol)
   }
-
+  
   return(res)
 }
 
@@ -768,6 +782,7 @@ EMM.cpp <- function(y, X = NULL, ZETA, eigen.G = NULL, eigen.SGS = NULL, n.thres
 #' @param optimizer The function used in the optimization process. We offer "optim", "optimx", and "nlminb" functions.
 #' @param traceInside Perform trace for the optimzation if traceInside >= 1, and this argument shows the frequency of reports.
 #' @param n.thres If \eqn{n >= n.thres}, perform EMM1.cpp. Else perform EMM2.cpp.
+#' @param n.core Setting n.core > 1 will enable parallel execution on a machine with multiple cores.
 #' @param tol The tolerance for detecting linear dependencies in the columns of G = ZKZ'.
 #' Eigen vectors whose eigen values are less than "tol" argument will be omitted from results.
 #' If tol is NULL, top 'n' eigen values will be effective.
@@ -800,29 +815,29 @@ EMM.cpp <- function(y, X = NULL, ZETA, eigen.G = NULL, eigen.SGS = NULL, n.thres
 #'
 #'
 #'
-EM3.cpp <- function (y, X0 = NULL, ZETA, eigen.G = NULL, eigen.SGS = NULL, tol = NULL,
+EM3.cpp <- function (y, X0 = NULL, ZETA, eigen.G = NULL, eigen.SGS = NULL, tol = NULL, n.core = NA,
                      optimizer = "nlminb", traceInside = 0, n.thres = 450, REML = TRUE, pred = TRUE){
   n <- length(as.matrix(y))
   y <- matrix(y, n, 1)
-
+  
   not.NA <- which(!is.na(y))
-
+  
   if (is.null(X0)) {
     p <- 1
     X0 <- matrix(rep(1, n), n, 1)
   }
   p <- ncol(X0)
-
+  
   lz <- length(ZETA)
   weights <- rep(1/lz, lz)
-
+  
   if(lz >= 2){
     if(is.null(eigen.G)){
       Z <- c()
       ms <- rep(NA, lz)
       for (i in 1:lz) {
         Z.now <- ZETA[[i]]$Z
-
+        
         if(is.null(Z.now)){
           Z.now <- diag(n)
         }
@@ -831,30 +846,30 @@ EM3.cpp <- function (y, X0 = NULL, ZETA, eigen.G = NULL, eigen.SGS = NULL, tol =
           m <- 1
           Z.now <- matrix(Z.now, length(Z.now), 1)
         }
-
+        
         K.now <- ZETA[[i]]$K
-
+        
         if (!is.null(K.now)) {
           stopifnot(nrow(K.now) == m)
           stopifnot(ncol(K.now) == m)
         }
-
+        
         Z <- cbind(Z, Z.now)
         ms[i] <- m
       }
-
+      
       stopifnot(nrow(Z) == n)
       stopifnot(nrow(X0) == n)
-
+      
       Z <- Z[not.NA, , drop = FALSE]
       X <- X0[not.NA, , drop = FALSE]
       n <- length(not.NA)
       y <- matrix(y[not.NA], n, 1)
-
+      
       spI <- diag(n)
       S <- spI - tcrossprod(X %*% solve(crossprod(X)), X)
-
-
+      
+      
       minimfunctionouter <- function(weights = rep(1 / lz, lz)) {
         weights <- weights / sum(weights)
         ZKZt <- matrix(0, nrow = n, ncol = n)
@@ -862,13 +877,13 @@ EM3.cpp <- function (y, X0 = NULL, ZETA, eigen.G = NULL, eigen.SGS = NULL, tol =
           ZKZt <- ZKZt + weights[i] * tcrossprod(ZETA[[i]]$Z[not.NA, ] %*%
                                                    ZETA[[i]]$K, ZETA[[i]]$Z[not.NA, ])
         }
-
-
+        
+        
         res <- EM3_kernel(y, X, ZKZt, S, spI, n, p)
         lambda <- res$lambda
         eta <- res$eta
         phi <- res$phi
-
+        
         if(REML){
           minimfunc <- function(delta) {
             (n - p) * log(sum(eta ^ 2/{
@@ -882,18 +897,18 @@ EM3.cpp <- function (y, X0 = NULL, ZETA, eigen.G = NULL, eigen.SGS = NULL, tol =
             })) + sum(log(phi + delta))
           }
         }
-
+        
         optimout <- optimize(minimfunc, lower = 0, upper = 10000)
         return(optimout$objective)
       }
-
-
+      
+      
       parInit <- rep(1 / lz, lz)
       parLower <- rep(0, lz)
       parUpper <- rep(1, lz)
       traceNo <- ifelse(traceInside > 0, 3, 0)
       traceREPORT <- ifelse(traceInside > 0, traceInside, 1)
-
+      
       if (optimizer == "optim") {
         soln <- optim(par = parInit, fn = minimfunctionouter, control = list(trace = traceNo, REPORT = traceREPORT),
                       method = "L-BFGS-B", lower = parLower, upper = parUpper)
@@ -913,16 +928,16 @@ EM3.cpp <- function (y, X0 = NULL, ZETA, eigen.G = NULL, eigen.SGS = NULL, tol =
                       method = "L-BFGS-B", lower = parLower, upper = parUpper)
         weights <- soln$par
       }
-
+      
     }
   }
-
-
+  
+  
   Z <- c()
   ms <- rep(NA, lz)
   for (i in 1:lz) {
     Z.now <- ZETA[[i]]$Z
-
+    
     if(is.null(Z.now)){
       Z.now <- diag(n)
     }
@@ -931,18 +946,18 @@ EM3.cpp <- function (y, X0 = NULL, ZETA, eigen.G = NULL, eigen.SGS = NULL, tol =
       m <- 1
       Z.now <- matrix(Z.now, length(Z.now), 1)
     }
-
+    
     K.now <- ZETA[[i]]$K
-
+    
     if (!is.null(K.now)) {
       stopifnot(nrow(K.now) == m)
       stopifnot(ncol(K.now) == m)
     }
-
+    
     Z <- cbind(Z, Z.now)
     ms[i] <- m
   }
-
+  
   Z <- Z[not.NA, , drop = FALSE]
   if((lz == 1) | (!is.null(eigen.G))){
     X <- X0[not.NA, , drop = FALSE]
@@ -950,9 +965,9 @@ EM3.cpp <- function (y, X0 = NULL, ZETA, eigen.G = NULL, eigen.SGS = NULL, tol =
     y <- matrix(y[not.NA], n, 1)
     spI <- diag(n)
   }
-
+  
   weights <- weights / sum(weights)
-
+  
   ZKZt <- matrix(0, nrow = n, ncol = n)
   Klist <- NULL
   for(i in 1:lz){
@@ -967,9 +982,9 @@ EM3.cpp <- function (y, X0 = NULL, ZETA, eigen.G = NULL, eigen.SGS = NULL, tol =
   }
   K <- Matrix::.bdiag(Klistweighted)
   ZK <- as.matrix(Z %*% K)
-
-
-
+  
+  
+  
   if(is.null(eigen.G)){
     if(nrow(Z) <= n.thres){
       # return.SGS <- TRUE
@@ -980,45 +995,48 @@ EM3.cpp <- function (y, X0 = NULL, ZETA, eigen.G = NULL, eigen.SGS = NULL, tol =
     spectralG.res <- spectralG.cpp(ZETA = lapply(ZETA, function(x) list(Z = x$Z[not.NA, , drop = FALSE], K = x$K)),
                                    X = X, weights = weights, return.G = TRUE, return.SGS = return.SGS,
                                    tol = tol, df.H = NULL)
-
+    
     eigen.G <- spectralG.res[[1]]
     eigen.SGS <- spectralG.res[[2]]
   }
-
-  if(lz >= 2){
+  
+  if (lz >= 2) {
     ZETA.list <- list(A = list(Z = diag(n), K = ZKZt))
-    EMM.cpp.res <- EMM.cpp(y, X = X, ZETA = ZETA.list, eigen.G = eigen.G, eigen.SGS = eigen.SGS, traceInside = traceInside,
-                           optimizer = optimizer, tol = tol, n.thres = n.thres, return.Hinv = TRUE, REML = REML)
-  }else{
-    EMM.cpp.res <- EMM.cpp(y, X = X, ZETA = lapply(ZETA, function(x) list(Z = x$Z[not.NA, , drop = FALSE], K = x$K)), traceInside = traceInside,
-                           optimizer = optimizer, eigen.G = eigen.G, eigen.SGS = eigen.SGS, tol = tol, n.thres = n.thres, return.Hinv = TRUE, REML = REML)
+    EMM.cpp.res <- EMM.cpp(y, X = X, ZETA = ZETA.list, eigen.G = eigen.G, 
+                           eigen.SGS = eigen.SGS, n.core = n.core, 
+                           traceInside = traceInside, optimizer = optimizer, 
+                           tol = tol, n.thres = n.thres, return.Hinv = TRUE, REML = REML)
+  } else {
+    EMM.cpp.res <- EMM.cpp(y, X = X, ZETA = lapply(ZETA, function(x) list(Z = x$Z[not.NA, , drop = FALSE], K = x$K)),
+                           n.core = n.core, traceInside = traceInside, optimizer = optimizer, eigen.G = eigen.G,
+                           eigen.SGS = eigen.SGS, tol = tol, n.thres = n.thres, return.Hinv = TRUE, REML = REML)
   }
-
-
+  
+  
   Vu <- EMM.cpp.res$Vu
   Ve <- EMM.cpp.res$Ve
   beta <- EMM.cpp.res$beta
   LL <- EMM.cpp.res$LL
   Hinv <- EMM.cpp.res$Hinv
-
-
+  
+  
   e <- (y - {X %*% beta})
   Hinve <- Hinv %*% e
-
+  
   u <- crossprod(ZK, Hinve)
   Vinv <- (1 / Ve) * Hinv
   namesu <- c()
   for (i in 1:length(Klist)) {
     namesu <- c(namesu, paste("K", i, colnames(ZETA[[i]]$K), sep = "_"))
   }
-
+  
   rownames(u) <- namesu
   if(!pred){
     return(list(y.pred = NULL, Vu = Vu, Ve = Ve, beta = beta,
                 u = u, weights = weights, LL = LL, Vinv = Vinv, Hinv = Hinv))
   }else{
     n.all <- nrow(ZETA[[1]]$Z)
-
+    
     Z.all <- c()
     for (i in 1:lz) {
       if(is.null(Z.now)){
@@ -1028,14 +1046,14 @@ EM3.cpp <- function (y, X0 = NULL, ZETA, eigen.G = NULL, eigen.SGS = NULL, tol =
       }
       Z.all <- cbind(Z.all, Z.now)
     }
-
-
+    
+    
     if(ncol(X) != 1){
       y.pred <- c(X0 %*% beta) + c(Z.all %*% u)
     }else{
       y.pred <- rep(beta, n.all) + c(Z.all %*% u)
     }
-
+    
     return(list(y.pred = y.pred, Vu = Vu, Ve = Ve, beta = beta,
                 u = u, weights = weights, LL = LL, Vinv = Vinv, Hinv = Hinv))
   }
@@ -1081,6 +1099,7 @@ EM3.cpp <- function (y, X0 = NULL, ZETA, eigen.G = NULL, eigen.SGS = NULL, tol =
 #' The result of the eigen decompsition of \eqn{G = ZKZ'}. You can use "spectralG.cpp" function in RAINBOWR.
 #' If this argument is NULL, the eigen decomposition will be performed in this function.
 #' We recommend you assign the result of the eigen decomposition beforehand for time saving.
+#' @param n.core Setting n.core > 1 will enable parallel execution on a machine with multiple cores.
 #' @param tol The tolerance for detecting linear dependencies in the columns of G = ZKZ'.
 #' Eigen vectors whose eigen values are less than "tol" argument will be omitted from results.
 #' If tol is NULL, top 'n' eigen values will be effective.
@@ -1124,12 +1143,12 @@ EM3.cpp <- function (y, X0 = NULL, ZETA, eigen.G = NULL, eigen.SGS = NULL, tol =
 #'
 EM3.linker.cpp <- function (y0, X0 = NULL, ZETA = NULL, Zs0 = NULL, Ws0,
                             Gammas0 = lapply(Ws0, function(x) diag(ncol(x))), gammas.diag = TRUE,
-                            X.fix = TRUE, eigen.SGS = NULL, eigen.G = NULL, tol = NULL,
+                            X.fix = TRUE, eigen.SGS = NULL, eigen.G = NULL, n.core = NA, tol = NULL,
                             bounds = c(1e-06, 1e06), optimizer = "nlminb", traceInside = 0,
                             n.thres = 450, spectral.method = NULL, REML = TRUE, pred = TRUE){
   n0 <- length(as.matrix(y0))
   y0 <- matrix(y0, n0, 1)
-
+  
   if(is.null(ZETA)){
     Z0 <- diag(n0)
     K <- diag(n0)
@@ -1140,39 +1159,39 @@ EM3.linker.cpp <- function (y0, X0 = NULL, ZETA = NULL, Zs0 = NULL, Ws0,
     Z0 <- ZETA[[1]]$Z
     K <- ZETA[[1]]$K
   }
-
-
+  
+  
   if (is.null(X0)) {
     p <- 1
     X0 <- matrix(rep(1, n0), n0, 1)
   }
   p <- ncol(X0)
-
-
+  
+  
   not.NA <- which(!is.na(y0))
   n <- length(not.NA)
   y <- y0[not.NA]
   X <- X0[not.NA, , drop = FALSE]
   Z <- Z0[not.NA, , drop = FALSE]
-
+  
   ZETA <- list(A = list(Z = Z, K = K))
-
+  
   if(is.null(Zs0)){
     Zs0 <- lapply(Ws0, function(x) diag(nrow(x)))
   }
   Zs <- lapply(Zs0, function(x) x[not.NA, ])
-
+  
   Ws <- NULL
   for(i in 1:length(Ws0)){
     Ws.part <- list(Zs[[i]] %*% Ws0[[i]])
     Ws <- c(Ws, Ws.part)
   }
   lw <- length(Ws)
-
+  
   offset <- sqrt(n)
   spI <- diag(n)
-
-
+  
+  
   if(!REML){
     if(is.null(eigen.G)){
       eigen.G <- spectralG.cpp(ZETA = ZETA, X = X, return.G = TRUE,
@@ -1181,26 +1200,26 @@ EM3.linker.cpp <- function (y0, X0 = NULL, ZETA = NULL, Zs0 = NULL, Ws0,
     U <- eigen.G$U
     delta <- eigen.G$delta
   }
-
-
+  
+  
   if((!X.fix) | (is.null(eigen.SGS))){
     eigen.SGS <- spectralG.cpp(ZETA = ZETA, X = X, return.G = FALSE,
                                return.SGS = TRUE, tol = tol, df.H = NULL)[[2]]
   }
   U2 <- eigen.SGS$Q
   delta2 <- eigen.SGS$theta
-
-
+  
+  
   n.param <- lw + 1
   weights <- rep(1 / n.param, n.param + 1)
   weights[2:(n.param + 1)] <- weights[2:(n.param + 1)] / sum(weights[2:(n.param + 1)])
-
+  
   minimumfunctionouter <- function(weights){
     weights[2:(n.param + 1)] <- weights[2:(n.param + 1)] / sum(weights[2:(n.param + 1)])
-
+    
     lambda <- weights[1] / weights[2]
     gammas <- weights[-(1:2)] / weights[2]
-
+    
     Gammas <- NULL
     for(i in 1:lw){
       Gammas[[i]] <- Gammas0[[i]] * gammas[i]
@@ -1209,30 +1228,30 @@ EM3.linker.cpp <- function (y0, X0 = NULL, ZETA = NULL, Zs0 = NULL, Ws0,
     P.res <- P_calc(lambda, Ws, Gammas, U2, as.matrix(delta2), lw, TRUE, gammas.diag)
     P <- P.res$P
     lnP <- P.res$lnP
-
+    
     yPy <- c(crossprod(y, P %*% y))
-
+    
     if(REML){
       LL <- llik_REML(n, p, yPy, lnP)
     }else{
       Hinv.res <- P_calc(lambda, Ws, Gammas, U, as.matrix(delta), lw, FALSE, gammas.diag)
       Hinv <- Hinv.res$P
       lnHinv <- Hinv.res$lnP
-
+      
       LL <- llik_ML(n, yPy, lnHinv)
     }
-
+    
     obj <- -LL
     return(obj)
   }
-
-
+  
+  
   parInit <- rep(1 / n.param, n.param + 1)
   parLower <- rep(bounds[1], n.param + 1)
   parUpper <- c(bounds[2], rep(1, n.param))
   traceNo <- ifelse(traceInside > 0, 3, 0)
   traceREPORT <- ifelse(traceInside > 0, traceInside, 1)
-
+  
   if (optimizer == "optim") {
     soln <- optim(par = parInit, fn = minimumfunctionouter, control = list(trace = traceNo, REPORT = traceREPORT),
                   method = "L-BFGS-B", lower = parLower, upper = parUpper)
@@ -1252,12 +1271,12 @@ EM3.linker.cpp <- function (y0, X0 = NULL, ZETA = NULL, Zs0 = NULL, Ws0,
                   method = "L-BFGS-B", lower = parLower, upper = parUpper)
     weights <- soln$par[-1]
   }
-
-
+  
+  
   Zs.all.list <- c(list(K.A = Z), Zs)
   Zs0.all.list <- c(list(K.A = Z0), Zs0)
   ZWs <- NULL
-
+  
   Zs.all <- Z
   Zs0.all <- Z0
   weights <- weights / sum(weights)
@@ -1270,59 +1289,61 @@ EM3.linker.cpp <- function (y0, X0 = NULL, ZETA = NULL, Zs0 = NULL, Ws0,
     K.now <- tcrossprod(Ws0[[i]] %*% Gammas0[[i]], Ws0[[i]])
     Kweighted.now <- weights[i + 1] * tcrossprod(Ws0[[i]] %*% Gammas0[[i]], Ws0[[i]])
     ZKZt.now <-  weights[i + 1] * tcrossprod(Ws[[i]] %*% Gammas0[[i]], Ws[[i]])
-
+    
     Zs.all <- cbind(Zs.all, Z.now)
     Zs0.all <- cbind(Zs0.all, Z0.now)
-
+    
     ZWs.now <- list(list(Z = Z.now, W = Ws0[[i]], Gamma = Gammas0[[i]]))
     names(ZWs.now) <- names(Zs)[i]
     ZWs <- c(ZWs, ZWs.now)
-
+    
     Klist <- c(Klist, list(K.now))
     Klistweighted <- c(Klistweighted, list(Kweighted.now))
     ZKZt <- ZKZt + ZKZt.now
   }
-
+  
   K.all <- Matrix::.bdiag(Klistweighted)
   ZK <- as.matrix(Zs.all %*% K.all)
-
-
+  
+  
   ZETA.list <- list(A = list(Z = diag(n), K = ZKZt))
   spectralG.all <- spectralG.cpp(ZETA = ZETA, ZWs = ZWs, weights = weights, X = X, return.G = TRUE,
                                  return.SGS = TRUE, tol = tol, df.H = NULL, spectral.method = "eigen")
   eigen.G.all <- spectralG.all[[1]]
   eigen.SGS.all <- spectralG.all[[2]]
-
-  EMM.cpp.res <- EMM.cpp(y, X = X, ZETA = ZETA.list, eigen.G = eigen.G.all, eigen.SGS = eigen.SGS.all,
-                         optimizer = optimizer, traceInside = traceInside, n.thres = n.thres, return.Hinv = TRUE, REML = REML, tol = tol)
-
+  
+  EMM.cpp.res <- EMM.cpp(y, X = X, ZETA = ZETA.list, eigen.G = eigen.G.all,
+                         eigen.SGS = eigen.SGS.all, n.core = n.core,
+                         optimizer = optimizer, traceInside = traceInside, 
+                         n.thres = n.thres, return.Hinv = TRUE, REML = REML, tol = tol)
+  
   Vu <- EMM.cpp.res$Vu
   Ve <- EMM.cpp.res$Ve
   beta <- EMM.cpp.res$beta
   LL <- EMM.cpp.res$LL
   Hinv <- EMM.cpp.res$Hinv
-
+  
   e <- (y - {X %*% as.matrix(beta)})
   Hinve <- Hinv %*% e
-
+  
   u <- crossprod(ZK, Hinve)
   Vinv <- (1 / Ve) * Hinv
   namesu <- c()
   for (i in 1:length(Klist)) {
     namesu <- c(namesu, paste("K", i, colnames(Klist[[i]]), sep = "_"))
   }
-
+  
   rownames(u) <- namesu
-  if(!pred){
+  if (!pred) {
     return(list(y.pred = NULL, Vu = Vu, Ve = Ve, beta = beta,
                 u = u, weights = weights, LL = LL, Vinv = Vinv, Hinv = Hinv))
-  }else{
-    if(ncol(X) != 1){
+  } else {
+    if (ncol(X) != 1) {
       y.pred <- c(X0 %*% beta) + c(Zs0.all %*% u)
-    }else{
+    } else {
       y.pred <- rep(beta, n0) + c(Zs0.all %*% u)
     }
-
+    
     return(list(y.pred = y.pred, Vu = Vu, Ve = Ve, beta = beta,
                 u = u, weights = weights, LL = LL, Vinv = Vinv, Hinv = Hinv))
   }
